package com.namomedia.android;

import android.widget.TextView;

/**
 * An object representing an AdData returned from the server.
 *
 * @author nassar
 */
class AdDataImpl implements NamoAdData {
  private final BackendClient backendClient;

  private final String id;
  private final String advertiserName;
  private final String advertiserLogoUrl;
  private final String shortText;
  private final String mediumText;
  private final String longText;
  private final String imageUrl;
  private final String imageMetadata;
  private final String actionUrl;
  private final String impressionTrackingUrl;
  private final String clickTrackingUrl;
  private final NamoActionType actionType;

  private enum TextLength {
    SHORT,
    MEDIUM,
    LONG
  }

  /**
   * Use AdDataImpl.Builder instead of this constructor.
   */
  private AdDataImpl(BackendClient backendClient,
      String id,
      String advertiserName, String advertiserLogoUrl,
      String shortText, String mediumText, String longText,
      String imageUrl, String imageMetadata,
      String actionUrl, String impressionTrackingUrl,
      String clickTrackingUrl, NamoActionType actionType) {

    // TODO(nassar): Remove the circular dependency. We should pass only the callback
    // interface we need instead of all of backend client.

    this.backendClient = backendClient;
    this.id = id;
    this.advertiserName = advertiserName;
    this.advertiserLogoUrl = advertiserLogoUrl;

    this.shortText = shortText;
    this.mediumText = mediumText;
    this.longText = longText;

    this.imageUrl = imageUrl;
    this.imageMetadata = imageMetadata;
    this.actionUrl = actionUrl;
    this.impressionTrackingUrl = impressionTrackingUrl;
    this.clickTrackingUrl = clickTrackingUrl;
    this.actionType = actionType;
  }

  String getId() {
    return id;
  }

  @Override
  public NamoImageLoader loadImage() {
    return new ImageLoaderImpl(backendClient, imageUrl, imageMetadata);
  }

  @Override
  public NamoImageLoader loadAdvertiserIcon() {
    return new ImageLoaderImpl(backendClient, advertiserLogoUrl);
  }

  @Override
  public NamoTextLoader loadText() {
    String[] textValues = {shortText, mediumText, longText};
    return new TextLoaderImpl(textValues, new TextLoaderImpl.LoadListener() {
      @Override
      public void onTextLoaded(TextView textView, int emCount) {
        backendClient.logLayoutInfo("text", id, emCount, textView);
      }
    });
  }

  @Override
  public NamoTextLoader loadAdvertiserName() {
    String[] textValues = {advertiserName};
    return new TextLoaderImpl(textValues, /* Load listener */ null);
  }

  @Override
  public NamoActionType getActionType() {
    return actionType;
  }

  String getActionUrl() {
    return actionUrl;
  }

  String getImpressionTrackingUrl() {
    return impressionTrackingUrl;
  }

  String getClickTrackingUrl() {
    return clickTrackingUrl;
  }

  static class Builder {
    private final BackendClient backendClient;

    private String id;
    private String advertiserName;
    private String advertiserLogoUrl;
    private String shortText;
    private String mediumText;
    private String longText;
    private String imageUrl;
    private String imageMetadata;
    private String clickTargetUrl;
    private String impressionTrackingUrl;
    private String clickTrackingUrl;
    private NamoActionType actionType;

    Builder(BackendClient backendClient) {
      this.backendClient = backendClient;
    }

    public Builder setId(String id) {
      this.id = id;
      return this;
    }

    public Builder setAdvertiserName(String advertiserName) {
      this.advertiserName = advertiserName;
      return this;
    }

    public Builder setAdvertiserLogoUrl(String advertiserLogoUrl) {
      this.advertiserLogoUrl = advertiserLogoUrl;
      return this;
    }

    public Builder setShortText(String shortText) {
      this.shortText = shortText;
      return this;
    }

    public Builder setMediumText(String mediumText) {
      this.mediumText = mediumText;

      return this;
    }

    public Builder setLongText(String longText) {
      this.longText = longText;
      return this;
    }

    public Builder setImageUrl(String imageUrl) {
      this.imageUrl = imageUrl;
      return this;
    }

    public Builder setImageMetadata(String imageMetadata) {
      this.imageMetadata = imageMetadata;
      return this;
    }

    public Builder setClickTargetUrl(String clickTargetUrl) {
      this.clickTargetUrl = clickTargetUrl;
      return this;
    }

    public Builder setImpressionTrackingUrl(String impressionTrackingUrl) {
      this.impressionTrackingUrl = impressionTrackingUrl;
      return this;
    }

    public Builder setClickTrackingUrl(String clickTrackingUrl) {
      this.clickTrackingUrl = clickTrackingUrl;
      return this;
    }

    public Builder setActionType(NamoActionType actionType) {
      this.actionType = actionType;
      return this;
    }

    public AdDataImpl create() {
      return new AdDataImpl(backendClient, id, advertiserName, advertiserLogoUrl,
          shortText, mediumText, longText,
          imageUrl, imageMetadata, clickTargetUrl, impressionTrackingUrl,
          clickTrackingUrl, actionType);
    }
  }
}
