package com.namomedia.android;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Handler;
import android.view.View;
import android.view.ViewGroup;
import com.namomedia.android.volley.Response;
import com.namomedia.android.volley.VolleyError;

import java.lang.ref.WeakReference;
import java.util.HashSet;
import java.util.Set;
import java.util.WeakHashMap;

/**
 * Implementation for NamoListAdapter interface.
 */
class AdPlacerImpl implements NamoAdPlacer {
  protected final Context context;
  private final Handler impressionHandler;
  private final BackendClient backendClient;
  private final PositionAdjusterImpl adjuster;
  private final WeakHashMap<View, AdDataImpl> adDataWeakMap;

  private final AdResponseListener adResponseListener;
  private final Response.ErrorListener errorListener;
  private final AdClickListener adClickListener;

  private NamoAdViewBinder adViewBinder;
  private NamoAdListener clientAdListener;

  private Set<String> viewedAdIds;
  private JsonRequest pendingRequest;
  private boolean isInitialRequest;
  private String continuationToken;
  private NamoTargeting continuationTargeting;

  AdPlacerImpl(Context context, PositionAdjusterImpl positionAdjuster,
      BackendClient backendClient) {
    this.context = context;
    this.impressionHandler = new Handler();
    this.backendClient = backendClient;
    this.adjuster = positionAdjuster;
    this.adViewBinder = new SimpleAdViewBinder(
        context, R.layout.namo_ad_view_sample_1, "namo-ad-view-sample-1");

    adDataWeakMap = new WeakHashMap<View, AdDataImpl>(8, 0.75f);
    viewedAdIds = new HashSet<String>();
    adResponseListener = new AdResponseListener();
    errorListener = new AdResponseErrorListener();
    adClickListener = new AdClickListener();
  }

  void clearPendingImpressions() {
    impressionHandler.removeMessages(0);
  }

  @Override
  public void registerAdViewBinder(NamoAdViewBinder adViewBinder) {
    this.adViewBinder = adViewBinder;
  }

  @Override
  public void setAdListener(NamoAdListener listener) {
    this.clientAdListener = listener;
  }

  @Override
  public void clearAdListener() {
    this.clientAdListener = null;
  }

  @Override
  public void requestAds() {
    requestAds(/* targeting */ null);
  }

  @Override
  public void requestAds(NamoTargeting targeting) {
    // Drop any outstanding requests.
    if (pendingRequest != null) {
      pendingRequest.cancel();
      pendingRequest = null;
    }

    isInitialRequest = true;
    requestAdsInternal(targeting, /* continuation token */ null, /* Min ad position */ 0);
    continuationTargeting = targeting;
  }

  private void requestMoreAds(int minAdPosition) {
    // Make sure no requests are outstanding and that we have a valid continuation token.
    if (pendingRequest != null || continuationToken == null) {
      return;
    }

    requestAdsInternal(continuationTargeting, continuationToken, minAdPosition);
  }

  private void requestAdsInternal(
      NamoTargeting targeting, String continuationToken, int minAdPosition) {
    // Don't allow a request while another is outstanding for this placer.
    Preconditions.checkState(pendingRequest == null);

    // The instance id is our best guess as to whether the user is in the same view.
    long viewId = System.identityHashCode(this);
    String formatId = adViewBinder.getFormatIdentifier();
    pendingRequest = backendClient.requestAds(formatId, targeting, viewId,
        continuationToken, minAdPosition,
        adResponseListener, errorListener);
  }

  @Override
  public NamoPositionAdjuster getPositionAdjuster() {
    return adjuster;
  }

  @Override
  public View placeAd(NamoAdData adData, View convertView, ViewGroup parent) {
    View view = convertView;
    if (view == null) {
      view = adViewBinder.createView(parent);
      view.setOnClickListener(adClickListener);
    }

    AdDataImpl adDataImpl = (AdDataImpl)adData;
    AdDataImpl oldAdData = adDataWeakMap.get(view);
    if (oldAdData != adDataImpl) {
      adDataWeakMap.put(view, adDataImpl);
      adViewBinder.bindAdData(view, adDataImpl);
      startImpressionTimer(view, adDataImpl);
    }

    // Request more ads if needed.
    NamoAdData[] ads = adjuster.getAds();
    int lastPos = ads.length - 1;
    if (lastPos >= 0 && ads[lastPos] == adData) {
      int minAdPosition = adjuster.getAdPositions()[lastPos] + 1;
      requestMoreAds(minAdPosition);
    }
    return view;
  }

  private void startImpressionTimer(final View view, final AdDataImpl adData) {
    if (viewedAdIds.contains(adData.getId())) {
      return;
    }

    // Don't allow the view to be kept alive if the activity is destroyed.
    final WeakReference<View> weakView = new WeakReference<View>(view);
    impressionHandler.postDelayed(new Runnable() {
      @Override
      public void run() {
        // If the view is collected, bail.
        View view = weakView.get();
        if (view == null) {
          return;
        }

        // Make sure we have the same ad data we did when the timer started.
        AdDataImpl currentAdData = adDataWeakMap.get(view);
        if (currentAdData == null || !currentAdData.getId().equals(adData.getId())) {
          return;
        }

        // TODO(nassar): We need to somehow also check that the view is still on the screen.

        // Prevent duplicate impressions.
        if (viewedAdIds.contains(adData.getId())) {
          return;
        }
        viewedAdIds.add(adData.getId());

        long viewId = System.identityHashCode(AdPlacerImpl.this);
        backendClient.logImpression(adData, viewId);
      }
    }, 1000);
  }

  private final class AdResponseErrorListener implements Response.ErrorListener {
    @Override
    public void onErrorResponse(VolleyError error) {
      pendingRequest = null;
    }
  }

  private final class AdResponseListener implements Response.Listener<AdResponse> {
    @Override
    public void onResponse(AdResponse response) {
      pendingRequest = null;

      if (isInitialRequest) {
        clearPendingImpressions();
        adjuster.clearAds();
        isInitialRequest = false;
      }

      adjuster.insertAds(response.ads, response.positioning.positions);
      continuationToken = response.positioning.continuationToken;
      if (clientAdListener != null) {
        clientAdListener.onAdsLoaded(adjuster);
      }
    }
  }

  private final class AdClickListener implements View.OnClickListener {

    @Override
    public void onClick(View view) {
      AdDataImpl adData = adDataWeakMap.get(view);
      if (adData == null) {
        return;
      }

      long viewId = System.identityHashCode(AdPlacerImpl.this);
      backendClient.logClick(adData, viewId);

      String actionUrl = adData.getActionUrl();
      if (actionUrl == null) {
        LogHelper.w("Ad contained an empty action URL.");
        return;
      }

      NamoActionType actionType = adData.getActionType();
      if (actionType == NamoActionType.INSTALL || actionType == NamoActionType.LINK) {
        Intent intent = new Intent(Intent.ACTION_VIEW);
        intent.setData(Uri.parse(actionUrl));
        context.startActivity(intent);
      } else if (actionType == NamoActionType.VIDEO) {
        Intent intent = new Intent(context, NamoAdActivity.class);
        intent.putExtra(NamoAdActivity.VIDEO_URL, actionUrl);
        context.startActivity(intent);
      }
    }
  }
}
