package com.namomedia.android;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.graphics.Point;
import android.os.Build;
import android.os.Handler;
import android.provider.Settings;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.widget.ImageView;
import android.widget.TextView;
import com.namomedia.android.volley.ParseError;
import com.namomedia.android.volley.RequestQueue;
import com.namomedia.android.volley.Response;
import com.namomedia.android.volley.VolleyError;
import com.namomedia.android.volley.toolbox.ImageLoader;
import com.namomedia.android.volley.toolbox.Volley;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.ref.WeakReference;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import java.util.WeakHashMap;

/**
 * Singleton client for connecting to Namo backend servers, including the ad server, click server,
 * and image server. This class is meant to be accessed from the UI thread.
 *
 * @author nassar
 */
class BackendClient {

  // TODO(nassar): Remove cyclic dependencies in this project (see IntelliJ inspections).

  private final SimpleDateFormat dateTimeFormatter =
      new SimpleDateFormat("yyyy'-'MM'-'dd'T'HH':'mm':'ss'Z'");
  private final SimpleDateFormat dateFormatter =
      new SimpleDateFormat("yyyy'-'MM'-'dd");

  private final Context appContext;
  private final RequestQueue requestQueue;
  private final ImageLoader volleyImageLoader;
  private final WeakHashMap<ImageView, ImageLoader.ImageContainer> imageRequestContainers;

  private final String deviceId;
  private final String deviceIdSha1;
  private final boolean isEmulator;
  private HashSet<String> testDeviceIds;
  private boolean isEmulatorInTestMode;

  private String adServerUrl = Urls.AD_SERVER_URL;

  private String imageServerUrl = Urls.IMAGE_SERVER_URL;
  // Cached request values.
  private final String sdkVersion;
  private final long startTimeMillis;
  private final String marketAppId;
  private final String marketAppVersion;
  private int sequenceNum;
  private final String userInfo;

  private final String carrier;
  private String applicationId;
  private boolean isActive;

  // This is a temporary hack to tack this on to the text impression URL. Once we build a
  // separate end point, this will go away.
  private JSONObject layoutTextInfo;

  // Private class to track information about the most recently viewed screen.
  private class ViewInfo {
    final long id;
    final long startTimeMillis;
    int sequenceNum;

    ViewInfo(long id, long startTimeMillis) {
      this.id = id;
      this.startTimeMillis = startTimeMillis;
      sequenceNum = 0;
    }
  }

  private ViewInfo currentViewInfo;

  BackendClient(Context appContext) {
    this.appContext = appContext;

    startTimeMillis = System.currentTimeMillis();

    // Initialize device ids.
    deviceId = Settings.Secure.getString(
        appContext.getContentResolver(), Settings.Secure.ANDROID_ID);
    deviceIdSha1 = sha1(deviceId);
    isEmulator = "google_sdk".equals(Build.PRODUCT)
        || "sdk".equals(Build.PRODUCT)
        || "sdk_x86".equals(Build.PRODUCT);
    testDeviceIds = new HashSet<String>();
    if (isEmulator) {
      LogHelper.i("The Namo SDK was initialized. It appears you are running in an emulator.");
    } else {
      LogHelper.i("The Namo SDK was initialized. Use Namo.setTestDevices(true, \"%s\");"
          + " to enable Test Mode.", deviceId);
    }

    // Setup other global info.
    marketAppId = appContext.getPackageName();
    marketAppVersion = getMarketAppVersion(appContext, marketAppId);
    sdkVersion = appContext.getResources().getString(R.string.namo_version);
    sequenceNum = 0;
    userInfo = "";
    carrier = getCarrier(appContext);

    // Initialize the network stack.
    requestQueue = Volley.newRequestQueue(appContext);
    ImageLoader.ImageCache imageCache = new BitmapMemCache();
    this.volleyImageLoader = new ImageLoader(requestQueue, imageCache);
    this.imageRequestContainers = new WeakHashMap<ImageView, ImageLoader.ImageContainer>();
  }

  private String getMarketAppVersion(Context context, String marketAppId) {
    PackageInfo info = null;
    try {
      info = context.getPackageManager().getPackageInfo(marketAppId, 0);
    } catch (PackageManager.NameNotFoundException e) {
      LogHelper.w("Unable to read package version");
    }
    if (info != null) {
      return info.versionName;
    }
    return "";
  }

  void setApplicationId(String applicationId) {
    this.applicationId = applicationId;
  }

  private String getCarrier(Context context) {
    TelephonyManager manager =
        (TelephonyManager)context.getSystemService(Context.TELEPHONY_SERVICE);
    return manager.getNetworkOperatorName();

  }

  /**
   * Enables or disables the client, allowing it to connect to the network.
   */
  void setActive(boolean active) {
    this.isActive = active;
    if (active) {
      requestQueue.start();
    } else {
      requestQueue.stop();
    }
  }

  /**
   * Returns whether the client is active.
   */
  boolean getActive() {
    return isActive;
  }

  /**
   * Log an impression for the given ad data.
   */
  void logImpression(AdDataImpl adData, long viewId) {
    logInteraction(adData.getImpressionTrackingUrl(), viewId);
  }

  /**
   * Log a click for the given ad data.
   */
  void logClick(AdDataImpl adData, long viewId) {
    logInteraction(adData.getClickTrackingUrl(), viewId);
  }

  /**
   * Log information about a textView
   */
  void logLayoutInfo(String fieldType, String adId, int emCount, TextView textView) {
    JSONObject textInfo = new JSONObject();
    try {
      textInfo.put("field_type", fieldType);
      textInfo.put("ad_id", adId);
      textInfo.put("em_count", emCount);
      FormatSerializer.appendTextViewProps(textInfo, textView);
    } catch (JSONException e) {
      LogHelper.w("Unable to send text view formatting information to the server.");
    }
    // Will be logged during the impression.
    layoutTextInfo = textInfo;
  }

  private void logInteraction(String actionUrl, long viewId) {
    // Don't log if the app isn't active
    if (applicationId == null || !isActive) {
      return;
    }

    RequestParams params = new RequestParams();
    appendCommonParams(params, viewId);
    appendLayoutInfoParams(params);

    Response.Listener<JSONObject> listener = new Response.Listener<JSONObject>() {
      @Override
      public void onResponse(JSONObject response) {
       LogHelper.d("Logged an impression");
      }
    };
    JsonRequest request = new JsonRequest(actionUrl, params, listener, defaultErrorListener);
    requestQueue.add(request);
  }

  /**
   * Request a single ad from the ad server.
   */
  JsonRequest requestAds(String formatType, NamoTargeting targeting, long viewId,
      String continuationToken, final int minAdPosition,
      final Response.Listener<AdResponse> listener, final Response.ErrorListener errorListener) {

    // The caller is probably not expecting an async callback, so make sure this isn't one.
    final Handler handler = new Handler();
    Response.ErrorListener wrappedErrorListener = new Response.ErrorListener() {
      @Override
      public void onErrorResponse(final VolleyError error) {
        handler.post(new Runnable() {
          @Override
          public void run() {
            defaultErrorListener.onErrorResponse(error);
            if (errorListener != null) {
              errorListener.onErrorResponse(error);
            }
          }
        });
      }
    };

    if (applicationId == null) {
      LogHelper.w("The Namo SKD does not have a valid application. Did you forget to call" +
          " Namo.initialize in your onCreate() method.");
      wrappedErrorListener.onErrorResponse(new VolleyError("Missing application ID"));
      return null;
    }
    if (!isActive) {
      LogHelper.w("The Namo SDK is currently deactivated. If you intentionally set it to inactive" +
          " and wish to get rid of this log message, you can check Namo.isActive before " +
          " requesting ads.");
      wrappedErrorListener.onErrorResponse(new VolleyError("Deactivated SDK"));
      return null;
    }

    // Build the URL params.
    RequestParams params = new RequestParams();
    appendCommonParams(params, viewId);
    appendRequestParams(params, formatType, continuationToken);
    if (targeting != null) {
      appendTargetingParams(params, targeting);
    }

    // Don't worry about making this async. Volley ensures that.
    Response.Listener<JSONObject> wrappedListener = new Response.Listener<JSONObject>() {
      @Override
      public void onResponse(JSONObject json) {
        AdResponse adResponse;
        try {
          adResponse = adResponseFromJson(json, minAdPosition);
        } catch (ParseError parseError) {
          errorListener.onErrorResponse(parseError);
          return;
        }
        listener.onResponse(adResponse);
      }
    };

    JsonRequest request = new JsonRequest(
        adServerUrl, params, wrappedListener, wrappedErrorListener);
    requestQueue.add(request);
    return request;
  }

  /**
   * Request an image from the image server, and load it into the target image view on completion.
   */
  void fetchSizedImage(String url, ImageView imageView,
      int placeholderResId, final int errorResId) {
    ImageLoader.ImageContainer currentRequest = imageRequestContainers.get(imageView);
    String currentUrl = (currentRequest == null) ? null : currentRequest.getRequestUrl();

    // If there was an old request in this view, check if it needs to be canceled.
    if (currentUrl != null) {
      if (currentUrl.equals(url)) {
        // If the request is from the same URL, return.
        return;
      } else {
        // If there is a pre-existing request, cancel it if it's fetching a different URL.
        currentRequest.cancelRequest();
        imageRequestContainers.remove(imageView);
      }
    }

    // If the new url is null, we're done.
    imageView.setImageResource(placeholderResId);
    if (TextUtils.isEmpty(url)) {
      return;
    }

    // Fetch the new image
    final WeakReference<ImageView> imageViewRef = new WeakReference<ImageView>(imageView);
    ImageLoader.ImageContainer request = volleyImageLoader.get(url, new ImageLoader.ImageListener() {
      @Override
      public void onResponse(ImageLoader.ImageContainer response, boolean isImmediate) {
        ImageView imageView = imageViewRef.get();
        if (imageView == null) {
          return;
        }
        imageRequestContainers.remove(imageView);

        if (isImmediate) {
          imageView.setImageBitmap(response.getBitmap());
        } else {
          CrossFadeDrawable.setBitmap(imageView, response.getBitmap());
        }
      }

      @Override
      public void onErrorResponse(VolleyError error) {
        ImageView imageView = imageViewRef.get();
        if (imageView == null) {
          return;
        }
        imageRequestContainers.remove(imageView);

        imageView.setImageResource(errorResId);
      }
    });
    imageRequestContainers.put(imageView, request);
  }

  private String sha1(String s) {
    // From http://www.androidsnippets.com/sha-1-hash-function
    MessageDigest digest;
    try {
      digest = MessageDigest.getInstance("SHA-1");
    } catch (NoSuchAlgorithmException e) {
      return null;
    }
    digest.reset();
    byte[] data = digest.digest(s.getBytes());
    return String.format("%0" + (data.length * 2) + "X", new BigInteger(1, data));
  }


  private AdResponse adResponseFromJson(JSONObject json, int minAdPosition) throws ParseError {
    try {
      JSONArray adsJson = json.getJSONArray("ads");
      JSONObject positioningJson = json.getJSONObject("positioning");
      JSONArray positionsJson = positioningJson.getJSONArray("positions");
      String continuationToken = positioningJson.optString("continuation_token", "");

      int numAds = adsJson.length();
      if (numAds != positionsJson.length()) {
        throw new ParseError(new Exception("Ad position mismatch with number of ads"));
      }

      AdDataImpl[] ads = new AdDataImpl[numAds];
      int[] positions = new int[numAds];
      for (int i = 0; i < numAds; i++) {
        int pos = positionsJson.getInt(i);

        // Don't allow overlapping ads.
        if (pos < minAdPosition) {
          throw new ParseError(new Exception("Ad positions not in sequential order"));
        }

        JSONObject adJson = adsJson.getJSONObject(i);
        ads[i] = adFromJson(adJson);
        positions[i] = pos;
        minAdPosition = pos + 1;
      }

      AdResponse.Positioning positioning = new AdResponse.Positioning(positions, continuationToken);
      return new AdResponse(ads, positioning);
    } catch (JSONException e) {
      throw new ParseError(e);
    }
  }

  private AdDataImpl adFromJson(JSONObject json) {
    AdDataImpl.Builder builder = new AdDataImpl.Builder(this);

    builder.setId(json.optString("ad_id"))
        .setAdvertiserName(json.optString("advertiser_name"))
        .setAdvertiserLogoUrl(json.optString("advertiser_logo"))
        .setShortText(json.optString("ad_short_text"))
        .setMediumText(json.optString("ad_medium_text"))
        .setLongText(json.optString("ad_long_text"))
        .setImageUrl(json.optString("ad_image"))
        .setImageMetadata(json.optString("ad_image_metadata"))
        .setClickTrackingUrl(json.optString("click_url"))
        .setClickTargetUrl(json.optString("click_target"))
        .setImpressionTrackingUrl(json.optString("impression_url"));

    String actionType = json.optString("action_type");
    builder.setActionType(Enums.valueIfPresent(NamoActionType.class, actionType));
    return builder.create();
  }

  private boolean isInTestMode() {
    if (isEmulator) {
      return isEmulatorInTestMode;
    }
    return testDeviceIds.contains(deviceId);
  }

  private void appendCommonParams(RequestParams params, long viewId) {
    long currentTimeMillis = System.currentTimeMillis();
    if (currentViewInfo == null || currentViewInfo.id != viewId) {
      currentViewInfo = new ViewInfo(viewId, currentTimeMillis);
    }

    params.put(Urls.Param.APP_ID, applicationId);
    params.put(Urls.Param.ANDROID_ID, deviceId);
    params.put(Urls.Param.TEST_MODE, isInTestMode() ? "1" : "0");
    params.put(Urls.Param.MARKET_APP_ID, marketAppId);
    params.put(Urls.Param.APP_VERSION, marketAppVersion);
    params.put(Urls.Param.USER_INFO, userInfo);
    params.put(Urls.Param.SEQUENCE, Long.toString(sequenceNum++));
    params.put(Urls.Param.CONNECTION_TYPE,
        Connectivity.getConnectionType(appContext).toUrlString());
    params.put(Urls.Param.SDK_VERSION, sdkVersion);

    params.put(Urls.Param.LOCAL_TIME, dateTimeFormatter.format(new Date(currentTimeMillis)));
    params.put(Urls.Param.TIME_SINCE_APP_OPENED, Double.toString(
        (currentTimeMillis - startTimeMillis) / 1000.0));

    params.put(Urls.Param.VIEW_ID, Long.toString(currentViewInfo.id));
    params.put(Urls.Param.VIEW_SEQUENCE, Integer.toString(currentViewInfo.sequenceNum++));
    params.put(Urls.Param.TIME_SINCE_VIEW_OPENED, Double.toString(
        (currentTimeMillis - currentViewInfo.startTimeMillis) / 1000.0));
  }

  private void appendLayoutInfoParams(RequestParams params) {
    // This is a hack to tack layout info on to the impression URL.
    // TODO(nassar): We should build another end-point for this.
    if (layoutTextInfo != null) {
      params.put(Urls.Param.LAYOUT_TEXT_INFO, layoutTextInfo.toString());
      layoutTextInfo = null;
    }
  }

  private void appendRequestParams(RequestParams params, String formatType,
      String continuationToken) {
    Point screenSize = Screen.getScreenSize(appContext);

    params.put(Urls.Param.CONTINUATION_TOKEN, continuationToken);

    params.put(Urls.Param.FORMAT_TYPE, formatType);
    params.put(Urls.Param.CARRIER, carrier);
    params.put(Urls.Param.SCREEN_WIDTH, Integer.toString(screenSize.x));
    params.put(Urls.Param.SCREEN_HEIGHT, Integer.toString(screenSize.y));
  }

  private void appendTargetingParams(RequestParams params, NamoTargeting targeting) {
    params.put(Urls.Param.GENDER, targeting.getGender().toString());
    params.put(Urls.Param.EDUCATION, targeting.getEducation().toString());
    params.put(Urls.Param.RELATION_STATUS, targeting.getRelationshipStatus().toString());
    if (targeting.getAge() != NamoTargeting.NO_AGE) {
      params.put(Urls.Param.AGE, Integer.toString(targeting.getAge()));
    }
    params.put(Urls.Param.SEARCH_TERMS, targeting.getSearchTerms());
    if (targeting.getBirthDay() != NamoTargeting.NO_BIRTHDAY) {
      params.put(Urls.Param.BIRTHDAY, dateFormatter.format(targeting.getBirthDay()));
    }

    params.putRepeatable(Urls.Param.INTEREST, targeting.getInterests());
    Set<String> keywords = targeting.getKeywords();
    if (!keywords.isEmpty()) {
      params.putRepeatable(Urls.Param.KEYWORD, keywords);
    }
  }

  /** A default error listener. There's not much we can do on error besides log. */
  private final Response.ErrorListener defaultErrorListener = new Response.ErrorListener() {
    @Override
    public void onErrorResponse(VolleyError error) {
      LogHelper.e("Network error on request" + error);
    }
  };

  void setTestDevices(boolean includeEmulator, String... deviceIds) {
    testDeviceIds.clear();
    Collections.addAll(testDeviceIds, deviceIds);
    isEmulatorInTestMode = includeEmulator;
  }

  void setDebugPref(String key, String value) {
    if ("ad_server_url".equals(key)) {
      adServerUrl = value != null ? value : Urls.AD_SERVER_URL;
    } else if ("image_server_url".equals(key)) {
      imageServerUrl = value != null ? value : Urls.IMAGE_SERVER_URL;
    }
  }

  String getDebugPref(String key) {
    if ("ad_server_url".equals(key)) {
      return adServerUrl;
    } else if ("image_server_url".equals(key)) {
      return imageServerUrl;
    }
    return null;
  }

  boolean isImageServerUrl(String url) {
    return url != null && url.startsWith(imageServerUrl);
  }
}
