package com.namomedia.android;

import android.graphics.Bitmap;
import com.namomedia.android.volley.toolbox.ImageLoader;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * An LruCache for bitmaps. This class is thread-safe.
 *
 * See http://developer.android.com/training/displaying-bitmaps/cache-bitmap.html. We can't use
 * LruCache because we support Froyo, so this is our own implementation.
 */
class BitmapMemCache implements ImageLoader.ImageCache {
  // Don't use soft references because in Gingerbread and earlier, bitmap memory is stored natively
  // and isn't correctly released.
  private final LinkedHashMap<String, Bitmap> map;

  // Sizes in bytes
  private long MAX_SIZE = Runtime.getRuntime().maxMemory() / 8;
  private long size;

  BitmapMemCache() {
    // Access order set to true so we can access elements in LRU order.
    this.map = new LinkedHashMap<String, Bitmap>(0, 0.75f, true);
  }

  @Override
  public synchronized Bitmap getBitmap(String url) {
    Preconditions.checkNotNull(url);
    return map.get(url);
  }

  @Override
  public void putBitmap(String url, Bitmap bitmap) {
    Preconditions.checkNotNull(url);
    Preconditions.checkNotNull(bitmap);

    synchronized (this) {
      size += sizeOf(bitmap);
      Bitmap previous = map.put(url, bitmap);
      if (previous != null) {
        size -= sizeOf(previous);
      }
    }

    // Trim to size. Use a separate synchronize block in each loop so that we don't block reading.
    while (true) {
      synchronized (this) {
        if (size <= MAX_SIZE || map.isEmpty()) {
          break;
        }

        Map.Entry<String, Bitmap> toEvict = map.entrySet().iterator().next();
        String key = toEvict.getKey();
        Bitmap value = toEvict.getValue();
        map.remove(key);
        size -= sizeOf(value);
      }
    }
  }

  private int sizeOf(Bitmap bitmap) {
    // Can't use getByteCount directly because it's only available in Honeycomb or greater.
    return bitmap.getRowBytes() * bitmap.getHeight();
  }
}
