package com.namomedia.android;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.ColorFilter;
import android.graphics.Rect;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.os.SystemClock;
import android.widget.ImageView;


/**
 * Ad drawable that allows cross-fading between a placeholder and an ad image bitmap.
 */
class CrossFadeDrawable extends Drawable {

  private static final float FADE_DURATION_MILLIS = 200f;

  /**
   * Create or update the drawable on the target {@link ImageView} to display the supplied bitmap
   * image.
   */
  static void setBitmap(ImageView imageView, Bitmap bitmap) {
    Drawable placeholder = imageView.getDrawable();
    CrossFadeDrawable drawable = new CrossFadeDrawable(imageView.getContext(), placeholder, bitmap);
    imageView.setImageDrawable(drawable);
  }

  private final BitmapDrawable image;
  private Drawable placeholder;
  private long startTimeMillis;
  private boolean animating;

  CrossFadeDrawable(Context context, Drawable placeholder, Bitmap bitmap) {
    Resources res = context.getResources();

    this.image = new BitmapDrawable(res, bitmap);

    this.placeholder = placeholder;
    animating = true;
    startTimeMillis = SystemClock.uptimeMillis();
  }

  @Override
  public void draw(Canvas canvas) {
    if (!animating) {
      image.draw(canvas);
    } else {
      float normalized = (SystemClock.uptimeMillis() - startTimeMillis) / FADE_DURATION_MILLIS;
      if (normalized >= 1f) {
        animating = false;
        placeholder = null;
        image.draw(canvas);
      } else {
        if (placeholder != null) {
          placeholder.draw(canvas);
        }

        int alpha = (int) (0xFF * normalized);
        image.setAlpha(alpha);
        image.draw(canvas);
        image.setAlpha(0xFF);
        invalidateSelf();
      }
    }
  }

  @Override
  public int getIntrinsicWidth() {
    return image.getIntrinsicWidth();
  }

  @Override
  public int getIntrinsicHeight() {
    return image.getIntrinsicHeight();
  }

  @Override
  public void setAlpha(int alpha) {
    if (placeholder != null) {
      placeholder.setAlpha(alpha);
    }
    image.setAlpha(alpha);
  }

  @Override
  public void setColorFilter(ColorFilter cf) {
    if (placeholder != null) {
      placeholder.setColorFilter(cf);
    }
    image.setColorFilter(cf);
  }

  @Override
  public int getOpacity() {
    return image.getOpacity();
  }

  @Override
  protected void onBoundsChange(Rect bounds) {
    super.onBoundsChange(bounds);

    image.setBounds(bounds);
    if (placeholder != null) {
      // Center placeholder inside the image bounds
      centerDrawable(placeholder);
    }
  }

  private void centerDrawable(Drawable drawable) {
    Rect bounds = getBounds();

    int width = bounds.width();
    int height = bounds.height();
    float ratio = (float) width / height;

    int drawableWidth = drawable.getIntrinsicWidth();
    int drawableHeight = drawable.getIntrinsicHeight();
    float drawableRatio = (float) drawableWidth / drawableHeight;

    if (drawableRatio < ratio) {
      float scale = (float) height / drawableHeight;
      int scaledDrawableWidth = (int) (drawableWidth * scale);
      int drawableLeft = bounds.left - (scaledDrawableWidth - width) / 2;
      int drawableRight = drawableLeft + scaledDrawableWidth;
      drawable.setBounds(drawableLeft, bounds.top, drawableRight, bounds.bottom);
    } else {
      float scale = (float) width / drawableWidth;
      int scaledDrawableHeight = (int) (drawableHeight * scale);
      int drawableTop = bounds.top - (scaledDrawableHeight - height) / 2;
      int drawableBottom = drawableTop + scaledDrawableHeight;
      drawable.setBounds(bounds.left, drawableTop, bounds.right, drawableBottom);
    }
  }
}