package com.namomedia.android;

import android.content.Context;
import android.database.DataSetObserver;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.BaseAdapter;
import android.widget.ListAdapter;

/**
 * Implementation for NamoListAdapter interface.
 * @param <T> The type of the original adapter.
 */
class ListAdapterImpl<T extends ListAdapter> extends BaseAdapter implements NamoListAdapter<T> {
  private final Context context;
  private final T originalAdapter;
  private final PositionAdjusterImpl adjuster;
  private final AdPlacerImpl adPlacer;

  private NamoAdListener clientAdListener;

  ListAdapterImpl(Context context, PositionAdjusterImpl positionAdjuster,
      final T originalAdapter, BackendClient backendClient) {
    this.context = context;
    this.adjuster = positionAdjuster;
    this.originalAdapter = originalAdapter;
    this.adPlacer = new AdPlacerImpl(context, positionAdjuster, backendClient);

    originalAdapter.registerDataSetObserver(new DataSetObserver() {
      @Override
      public void onChanged() {
        notifyDataSetChanged();
      }

      @Override
      public void onInvalidated() {
        notifyDataSetInvalidated();
      }
    });

    adPlacer.setAdListener(new NamoAdListener() {
      @Override
      public void onAdsLoaded(NamoPositionAdjuster adjuster) {
        notifyDataSetChanged();
        if (clientAdListener != null) {
          clientAdListener.onAdsLoaded(adjuster);
        }
      }
    });
  }

  @Override
  public void notifyDataSetChanged() {
    adPlacer.clearPendingImpressions();
    super.notifyDataSetChanged();
  }

  @Override
  public void registerAdLayout(int layoutResId, String formatIdentifier) {
    adPlacer.registerAdViewBinder(new SimpleAdViewBinder(context, layoutResId, formatIdentifier));
  }

  @Override
  public void registerAdViewBinder(NamoAdViewBinder adViewBinder) {
    adPlacer.registerAdViewBinder(adViewBinder);
  }

  @Override
  public void requestAds() {
    requestAds(null);
  }

  @Override
  public void requestAds(NamoTargeting targeting) {
    adPlacer.requestAds(targeting);
  }

  @Override
  public void setAdListener(NamoAdListener listener) {
    this.clientAdListener = listener;
  }

  @Override
  public void clearAdListener() {
    this.clientAdListener = null;
  }

  @Override
  public View placeAd(NamoAdData adData, View convertView, ViewGroup parent) {
    return adPlacer.placeAd(adData, convertView, parent);
  }

  @Override
  public NamoPositionAdjuster getPositionAdjuster() {
    return adjuster;
  }

  @Override
  public T getOriginalAdapter() {
    return originalAdapter;
  }

  @Override
  public int getCount() {
    return adjuster.getAdjustedPosition(originalAdapter.getCount() - 1) + 1;
  }

  @Override
  public int getViewTypeCount() {
    return originalAdapter.getViewTypeCount() + 1;
  }

  @Override
  public int getItemViewType(int position) {
    return adjuster.isAd(position) ? 0
        : originalAdapter.getItemViewType(adjuster.getOriginalPosition(position)) + 1;
  }

  @Override
  public Object getItem(int position) {
    AdDataImpl ad = adjuster.getAdData(position);
    return ad != null ? ad : originalAdapter.getItem(adjuster.getOriginalPosition(position));
  }

  @Override
  public long getItemId(int position) {
    return position;
  }

  @Override
  public boolean isEnabled(int position) {
    return adjuster.isAd(position) || originalAdapter.isEnabled(position);
  }

  @Override
  public boolean areAllItemsEnabled() {
    return originalAdapter.areAllItemsEnabled();
  }

  @Override
  public View getView(int position, View convertView, ViewGroup parent) {
    AdDataImpl adData = adjuster.getAdData(position);
    if (adData == null) {
      int originalPosition = adjuster.getOriginalPosition(position);
      return originalAdapter.getView(originalPosition, convertView, parent);
    }

    return adPlacer.placeAd(adData, convertView, parent);
  }

  @Override
  public OnItemClickListener createWrappedClickListener(final OnItemClickListener originalListener) {
    return new OnItemClickListener() {
      @Override
      public void onItemClick(AdapterView<?> adapterView, View view, int position, long id) {
        if (!adjuster.isAd(position) && originalListener != null) {
          originalListener.onItemClick(adapterView, view,
              adjuster.getOriginalPosition(position), id);
        }
      }
    };
  }

  @Override
  public OnItemLongClickListener createWrappedLongClickListener(
      final OnItemLongClickListener originalListener) {
    Preconditions.checkNotNull(originalListener);

    return new OnItemLongClickListener() {
      @Override
      public boolean onItemLongClick(AdapterView<?> adapterView, View view, int position, long id) {
        if (!adjuster.isAd(position)) {
          return originalListener.onItemLongClick(adapterView, view,
              adjuster.getOriginalPosition(position), id);
        }

        // TODO(nassar): In test mode, show ad debugging information.

        return true;
      }
    };
  }

  @Override
  public OnItemSelectedListener createWrappedSelectedListener(
      final OnItemSelectedListener originalListener) {
    Preconditions.checkNotNull(originalListener);

    return new OnItemSelectedListener() {
      @Override
      public void onItemSelected(AdapterView<?> adapterView, View view, int position, long id) {
        if (!adjuster.isAd(position)) {
          originalListener.onItemSelected(
              adapterView, view, adjuster.getOriginalPosition(position), id);
        }
      }

      @Override
      public void onNothingSelected(AdapterView<?> adapterView) {
        originalListener.onNothingSelected(adapterView);
      }
    };
  }
}
