// Copyright (c) 2013 Namo Media, Inc. All rights reserved.

package com.namomedia.android;

import android.content.Context;
import android.widget.ListAdapter;

/**
 * Static functions to handle global configuration of the Namo SDK.
 *
 * @since 1.0
 */
public final class Namo {

  private static volatile BackendClient sBackendClient;

  private Namo() {
    // Non-instantiable.
  }

  /**
   * Authenticates that this client belongs to your application.
   * <p>
   * You must invoke this method in order to use the Namo SDK. This method is idempotent and has
   * negligible runtime overhead, so you can call it multiple times if convenient.
   * <p>
   * We recommend initializing in your Application.onCreate method, or in the Activity or Fragment
   * where you use the SDK. If you use the SDK from multiple Activities and don't want to use
   * Application.onCreate, make sure you initialize the SDK in every activity where you'll use the
   * SDK. Otherwise Android may kill your application and restore just the top most activity.
   *
   * @param context An application or activity context. If passed an activity context, the SDK will
   *     still use the parent application context in order to prevent a memory leak.
   * @param applicationId The application ID for your registered application, found at <a
   *     href="https://dashboard.namomedia.com/monetize" target="_blank">dashboard.namomedia.com</a>.
   * @since 1.0
   */
  public static void initialize(Context context, String applicationId) {
    Preconditions.checkNotNull(
        context, "You must initialize the SDK with a non-null context");
    Preconditions.checkNotNull(
        applicationId, "You must initialize the SDK with a non-null application id");

    // Initialize the backend client just once.
    if (sBackendClient == null) {
      synchronized (Namo.class) {
        if (sBackendClient == null) {
          sBackendClient = new BackendClient(context.getApplicationContext());
        }
      }
    }
    sBackendClient.setApplicationId(applicationId);
    sBackendClient.setActive(true);
  }

  /**
   * Enables or disables the SDK from connecting to the server for ads.
   * <p>
   * When not active, calling {@link com.namomedia.android.NamoListAdapter#requestAds} will log a
   * warning message. To prevent this warning, you can check the active value before requesting ads.
   *
   * @param active <code>true</code> to enable ads, <code>false</code> to disable serving ads.
   * @since 1.0
   */
  public static void setActive(boolean active) {
    if (!hasBackendClient()) {
      return;
    }
    sBackendClient.setActive(active);
  }

  /**
   * Returns whether the SDK is currently active.
   * <p>
   * Defaults to <code>true</code> unless you have explicitly called {@link #setActive}.
   *
   * @return Whether the SDK is active.
   * @since 1.0
   */
  public static boolean isActive() {
    return hasBackendClient() && sBackendClient.getActive();
  }

  /**
   * Creates and returns an adapter that wraps the original adapter and inserts ads into your
   * stream.
   * <p>
   * Set this adapter to your ListView using {#setAdapter} to insert ads. For non-ad items, this
   * adapter will call your original adapter with the original row positions before inserting ads.
   * <p>
   * The returned adapter also implements the {@link com.namomedia.android.NamoAdPlacer} interface,
   * which you should use to request ads. The adapter will call {#notifyDataSetChanged} once ads
   * are available.
   *
   * @param context The activity context for this adapter.
   * @param originalAdapter Your original ListView adapter.
   * @return The newly created NamoListAdapter.
   * @since 1.0
   */
  public static <T extends ListAdapter> NamoListAdapter<T> createListAdapter(
      Context context, T originalAdapter) {
    Preconditions.checkNotNull(
        originalAdapter, "Namo#createListAdapter: getOriginalAdapter cannot be null");
    Preconditions.checkState(hasBackendClient(),
        "Namo#createListAdapter: You must call Namo#initialize before creating a list adapter");
    PositionAdjusterImpl adjuster = new PositionAdjusterImpl();
    return new ListAdapterImpl<T>(context, adjuster, originalAdapter, sBackendClient);
  }

  /**
   * Creates and returns an ad placer for inserting ads into your stream.
   *
   * @param context The activity context for the placer.
   * @return The newly created NamoAdPlacer.
   * @since 1.1.0
   */
  public static NamoAdPlacer createAdPlacer(Context context) {
    Preconditions.checkState(hasBackendClient(),
        "Namo#createAdPlacer: You must call Namo#initialize before creating a stream adapter");
    PositionAdjusterImpl adjuster = new PositionAdjusterImpl();
    return new AdPlacerImpl(context, adjuster, sBackendClient);
  }

  /**
   * Puts the given devices into 'test' mode.
   * <p>
   * Test mode ensures that ads will always be shown. It also will prevent impressions and clicks
   * from the device from being counted as revenue. We recommend adding any devices you use for
   * development.
   * <p>
   * You test device ID is your device ID. You can easily obtain this value by copying and pasting
   * them from the logs printed by the SDK when you first call {@link #initialize}.
   *
   * @param includeEmulator Whether to include Android emulators as test devices.
   * @param deviceIds An array of UUIDs containing test device IDs. You do not need to pass an
   * ID for your emulator. Use the 'includeEmulator' parameter instead.
   * @since 1.1.0
   */
  public static void setTestDevices(boolean includeEmulator, String... deviceIds) {
    Preconditions.checkState(hasBackendClient(),
        "Namo#createListAdapter: You must call Namo#initialize before setting test devices");
    sBackendClient.setTestDevices(includeEmulator, deviceIds);
  }

  /**
   * Sets a debugging pref. Not intended to be public, but can be called outside this package using
   * reflection.
   */
  static void setDebugPref(String key, String value) {
    Preconditions.checkNotNull(key);
    if (!hasBackendClient()) {
      return;
    }
    sBackendClient.setDebugPref(key, value);
  }

  /**
   * Sets a debugging pref. Not intended to be public, but can be called outside this package using
   * reflection.
   */
  static String getDebugPref(String key) {
    Preconditions.checkNotNull(key);
    if (!hasBackendClient()) {
      return "";
    }
    return sBackendClient.getDebugPref(key);
  }

  private static boolean hasBackendClient() {
    if (sBackendClient == null) {
      LogHelper.w("The Namo SDK is not initialized. Did you forget to call Namo.initialize?");
      return false;
    }
    return true;
  }
}
