package com.namomedia.android;

import android.view.View;
import android.view.ViewGroup;

/**
 * This interface allows you to request ads and place them into your stream.
 * <p>
 * Instantiate an instance of this class by calling
 * {@link com.namomedia.android.Namo#createAdPlacer}. If you are inserting ads into a ListView,
 * you may want to create a {@link com.namomedia.android.NamoListAdapter} object instead,
 * which extends this interface.
 * <p>
 * To request ads, call the {@link #requestAds} method. When requesting ads, you should pass
 * targeting information to increase the chance that you show ads that are relevant to your users.
 * <p>
 * This class is not designed to be used by multiple threads. All calls should be made from the
 * main UI thread.
 *
 * @since 1.1.0
 */
public interface NamoAdPlacer {

  /**
   * Registers a binder object that allows you to create and bind your own ad view class.
   * <p>
   * Use this method to control how you ad list item is created or bound to ad data. For example,
   * you can use this method to register a binder that inflates your regular list item layout
   * and then binds data using
   * {@link com.namomedia.android.NamoAdViewBinder#bindAdData}.
   *
   * @param adViewBinder The ad view binder for creating your ad view and binding ad content.
   * @since 1.0
   */
  void registerAdViewBinder(NamoAdViewBinder adViewBinder);

  /**
   * Request ads from the server. We recommend using {@link #requestAds(NamoTargeting)} instead
   * of this method.
   *
   * @since 1.0
   */
  void requestAds();

  /**
   * Requests ads from the server.
   * <p>
   * We recommend requesting ads in your onResume method so that each visit to your stream will
   * result in a new set of ad impressions. You may also want to request ads after your user
   * performs an action that results in new stream items.
   *<p>
   * Implementations of this class that are based on Android's list
   * adapter should also call {#notifyDataSetChanged} to ensure that the ads are immediately
   * displayed.
   * <p>
   * If you call requestAds on the same placer more than once, any existing ads will be cleared and
   * a new set of ads will be loaded from the server. If there is already a pending request to the
   * server, that request will be cancelled.
   *
   * @param targeting Targeting information to send in the ad request.
   * @since 1.0
   */
  void requestAds(NamoTargeting targeting);

  /**
   * Sets a listener that will be called when after the SDK loads new ads from the server.
   * <p>
   * Of note, there is not a one to one correspondence between calls to requestAds and your
   * your listener. To support continuous scrolling, the SDK will fetch additional ads as your user
   * moves forward through your stream, and will call your callback every time new ads are
   * retrieved. If there is an error fetching ads, such as a network timeout, the listener will
   * not be called.
   * <p>
   * This listener is always called asynchronously with respect to requestAds
   *
   * @param listener Listener that will be called when the SDK loads new ads.
   * @since 2.0
   */
  void setAdListener(NamoAdListener listener);

  /**
   * Clears the listener set by {#setAdListener}.
   *
   * @since 2.0
   */
  void clearAdListener();

  /**
   * Places the given ad data into a view, creating the view if it does not exist.
   * <p>
   * Calling this method will invoke one of your registered
   * {@link com.namomedia.android.NamoAdViewBinder} objects. If convertView is <code>null</code>,
   * this method will call {@link com.namomedia.android.NamoAdViewBinder#createView} to
   * instantiate a new instance, followed by a call to
   * {@link com.namomedia.android.NamoAdViewBinder#bindAdData}.
   * <p>
   * You should use this method instead of your own ad view rendering mechanism so that the SDK can
   * attach click and impression tracking handlers.
   *
   * @param adData The ad data. Should not be null.
   * @param convertView A recycled view into which to bind data, or <code>null</code>.
   * @param parent The parent that the view will eventually be attached to. You might use the parent
   * to determine layout parameters, but should return the view without attaching it to the
   * parent.
   * @return The newly placed ad view. If convertView param was not <code>null</code>, this will
   * be a reference to convertView.
   * @since 1.1.0
   */
  View placeAd(NamoAdData adData, View convertView, ViewGroup parent);

  /**
   * Returns the index path adjuster for this list adapter.
   * <p>
   * The adjuster is responsible for translating
   * between your original data source positions and the new list item positions.
   * {@see com.namomedia.android.NamoIndexPathAdjuster}
   *
   * @return The current index path adjuster.
   * @since 1.0
   */
  NamoPositionAdjuster getPositionAdjuster();
}
