package com.namomedia.android;

import android.view.View;
import android.view.ViewGroup;

/**
 * An interface for creating ad views and binding them to ad data.
 *
 * Normally, you will use {@link com.namomedia.android.NamoListAdapter#registerAdLayout} to
 * customize your ad cell with your own layout. However, if you wish to programmatically create
 * or manage your ad cell, you can implement a <code>NamoAdViewBinder</code> object and call
 * {@link com.namomedia.android.NamoListAdapter#registerAdViewBinder} instead.
 *
 * @since 1.0
 */
public interface NamoAdViewBinder {

  /**
   * Returns a unique format identifier for this list item.
   * <p>
   * This identifier is sent to the Namo ad server in order to record statistics about your ad
   * format for better insight into how different formats perform. It may also be used at ad
   * request time to rank ads that are better suited for the format based on information the
   * SDK is able to derive about the format layout. For example, if your format contains large
   * images the server will prefer ads that are suited to be displayed as large photos.
   * <p>
   * Your format identifier can be whatever name you wish, but should be unique for each ad
   * cell layout used by your app.
   *
   * @return The format identifier.
   * @since 1.0
   */
  String getFormatIdentifier();

  /**
   * Creates a new view to be used as an ad list item.
   * <p>
   * This method is called by {@link com.namomedia.android.NamoListAdapter#getView} when the
   * convertView is null. You must return a valid view.
   *
   * @param parent The parent that the view will eventually be attached to. You might use the parent
   * to determine layout parameters, but should return the view without attaching it to the
   * parent.
   * @return An ad list item.
   * @since 1.0
   */
  View createView(ViewGroup parent);

  /**
   * Binds a view created in {@link #createView} to ad data.
   * <p>
   * This method is called by {@link com.namomedia.android.NamoListAdapter#getView}. When
   * implementing this method, you may consider using the ViewHolder pattern to ensure smooth
   * list scrolling performance.
   * {@see http://developer.android.com/training/improving-layouts/smooth-scrolling.html}
   *
   * @param view The list item view.
   * @param adData The ad data that should be bound to the view.
   * @since 1.0
   */
  void bindAdData(View view, NamoAdData adData);
}
