package com.namomedia.android;

import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ListAdapter;

/**
 * An adapter that wraps an underlying Android ListAdapter and insert ads into a list stream. Create
 * a new instance of this adapter using {@link com.namomedia.android.Namo#createListAdapter}, and
 * set it as the adapter for your ListView.
 * <p/>
 * This interface extends {@link com.namomedia.android.NamoAdPlacer}, which provides methods for
 * requesting ads and controlling their placement in your list.
 *
 * @param <T> The type of the wrapped original adapter.
 * @since 1.0
 */
public interface NamoListAdapter<T extends ListAdapter> extends ListAdapter, NamoAdPlacer {

  /**
   * Registers an Android layout resource to inflated when displaying ads.
   * <p>
   * This layout will be automatically inflated the first time getView is called on an AdView.
   * Subsequently, the inflated view will be recycled using the normal ListView recycling
   * mechanism. If you wish to control how your view gets initialized, you should use
   * {@link #registerAdViewBinder} instead.
   * <p>
   * Your layout should contain fields whose ids are set to the
   * <code>com.namomedia.android.R.id</code> constants in order to
   * show ad content in your view. You should also pass a format identifier so that you see metrics
   * on how users interacted with this layout.
   *<p>
   * If you register multiple ad layouts, the SDK will automatically choose which layout to use
   * based on which layout best matches the ad being shown.
   *
   * @param layoutResId The Android layout resource id to use for this ad.
   * @param formatIdentifier The string identifier for this ad format.
   * @since 1.0
   */
  void registerAdLayout(int layoutResId, String formatIdentifier);

  /**
   * Notifies the ListView that the underlying data has been changed.
   * <p>
   * Calling this method is
   * equivalent to calling notifyDataSetChanged on your original adapter. This method is
   * provided as a convenience method.
   *
   * @since 1.0
   */
  void notifyDataSetChanged();

  /**
   * Notifies the ListView that the underlying data has been changed.
   * <p>
   * Calling this method is
   * equivalent to calling notifyDataSetChanged on your original adapter. This method is
   * provided as a convenience method.
   *
   * @since 1.0
   */
  void notifyDataSetInvalidated();

  /**
   * Returns the original adapter currently being wrapped by this adapter.
   *
   * @return The original adapter.
   * @since 1.0
   */
  T getOriginalAdapter();

  /**
   * Returns an {@link OnItemClickListener} that also handles ad clicks.
   *
   * @param originalListener the click listener for non-ad items. May be null.
   * @return A new click listener that forwards requests to the provided listener.
   * @since 1.0
   */
  OnItemClickListener createWrappedClickListener(OnItemClickListener originalListener);

  /**
   * Returns an {@link OnItemLongClickListener} that also handles ad clicks.
   *
   * @param originalListener the click listener for non-ad items. May be null.
   * @return A new long click listener that forwards requests to the provided listener.
   * @since 1.0
   */
  OnItemLongClickListener createWrappedLongClickListener(OnItemLongClickListener originalListener);

  /**
   * Returns an {@link OnItemSelectedListener} that also handles ad selection.
   *
   * @param originalListener the selection listener for non-ad items. May be null.
   * @return A new long click listener that forwards requests to the provided listener.
   * @since 1.0
   */
  OnItemSelectedListener createWrappedSelectedListener(OnItemSelectedListener originalListener);
}
