package com.namomedia.android;

/**
 * Adjusts the positions of your list items in order to insert ads.
 * <p>
 * If you are using {@link com.namomedia.android.NamoListAdapter} to wrap your own adapter,
 * the positioning logic is taken care of for you. You should typically use this class in
 * coordination with {@link com.namomedia.android.NamoAdPlacer}.
 * <p>
 * Some examples of using this class:
 * <ol>
 *   <li>You've decided not to use NamoListAdapter to wrap your adapter, and are using
 *   {@link com.namomedia.android.NamoAdPlacer} directly from your own adapter. You've registered
 *   a {@link com.namomedia.android.NamoAdViewBinder} with the placer and requested ads. In your
 *   getView method, you can use {@link #isAd} to check which positions are ads and place them
 *   accordingly:
 *   <pre><code>   {@literal @}Override
 *   public View getView(int position, View convertView, ViewGroup parent) {
 *     if (adjuster.isAd(position)) {
 *       return adPlacer.placeAdView(adjuster.getAdData(position), convertView, parent);
 *     }
 *     ExampleItem item = items.get(streamHelper.getOriginalPosition(i));
 *     ...
 *   }</code></pre>
 *   </li>
 *
 *   <li>As in the last example, you are using a NamoAdPlacer directly. You've decided that you
 *   also want your adapter's getItem method to return a
 *   {@link com.namomedia.android.NamoAdData} object for items
 *   that contain ads. You can use {@link #getAdData} and check whether the return
 *   value is null.
 *   <pre><code>   {@literal @}Override
 *   public Object getItem(int position) {
 *     NamoAdData adData = adjuster.getAdData(position);
 *     if (adData != null) {
 *       return adData;
 *     }
 *     return items.get(adjuster.getOriginalPosition(i));
 *   }
 *
 *   {@literal @}Override
 *   public View getView(int position, View convertView, ViewGroup parent) {
 *     Object item = getItem(position);
 *     if (item instanceof NamoAdData) {
 *       return streamHelper.placeAdView((NamoAdData)item, convertView, parent);
 *     }
 *     ...
 *   }</code></pre>
 *
 *   <li>You are inserting ads into your stream, and are receiving an item click callback
 *   (which passes a position) from your ListView. You want to use the position to look up
 *   an item in a backing data source. Since the position you've received from the event is the
 *   position <i>after inserting ads</i>, you need to use {@link #getOriginalPosition} to find the
 *   original position.
 *   <p>
 *   <b>Note:</b> If you were using NamoListAdapter, you can just use
 *   {@link com.namomedia.android.NamoListAdapter#createWrappedClickListener} to create your click
 *   listener. You still may need to find the original position after calling a method like
 *   ListView.getFirstVisiblePosition.</li>
 *
 *   <li>You have the position of an item from your backing data source, and you want to use that
 *   position to find an item in your list. You need to use {@link #getAdjustedPosition} to
 *   get the adjusted list item position taking into account the currently inserted ads.
 *   <p>
 *   <b>Note:</b> If you are using NamoListAdapter, getItem will still return your original item.
 *   You still may need to adjust your position before calling a method like
 *   ListView.setSelection.</li>
 * </ol>
 * <p>
 * Manual position adjusting is an experimental feature and the API is subject to change. For help
 * or suggestions, feel free to reach out to us at
 * <a href="mailto:contact@namomedia.com">contact@namomedia.com</a>.
 * @since 1.1.0
 */
public interface NamoPositionAdjuster {

  /**
   * Returns the position or an item adjusted based on ads in the list.
   *
   * @param position the original position.
   * @return The position adjusted by ad placement in the stream.
   * @since 1.0
   */
  int getAdjustedPosition(int position);

  /**
   * Returns the original position in your adapter given an adjusted position.
   * <p>
   * If the given position is currently showing an ad, returns -1.
   *
   * @param adjustedPosition The adjusted position.
   * @return The original position.
   * @since 1.0
   */
  int getOriginalPosition(int adjustedPosition);

  /**
   * Returns whether the item at given position is an ad.
   *
   * @param adjustedPosition The adjusted position.
   * @return Whether the item at the given position is an ad.
   * @since 1.0
   */
  boolean isAd(int adjustedPosition);

  /**
   * Returns the ad at the given position.
   *
   * @param adjustedPosition The adjusted position.
   * @return The ad at the given position, or <code>null</code> if the item at the position is not
   *     an ad.
   * @since 1.0
   */
  NamoAdData getAdData(int adjustedPosition);

  /**
   * Returns the number of rows in the adapter adjusted for the ads that will appear in the list.
   *
   * @param originalCount The original number of rows in the stream.
   * @return The adjusted number of rows in the list.
   * @since 1.1.0
   */
  int getAdjustedCount(int originalCount);

  /**
   * Returns the original number of items to display before inserting ads.
   * <p>
   * Your can pass this function the value returned by NamoListAdapter's
   * <code>getCount()</code> method to get the count from your original data source.
   *
   * @param adjustedCount The adjusted number of items in the stream including ads.
   * @return The original number of items.
   * @since 1.1.0
   */
  int getOriginalCount(int adjustedCount);

  /**
   * Removes the content row at the given position, adjusting ad positions accordingly.
   *
   * This method is useful if you are removing items from your underlying data source and
   * wish to move the positions of the ads rather than keeping them at their current positions
   * relative to the new data.
   * <p>
   * You can also use this method to remove an ad at the given position.
   *
   * @param position The position at which to remove an item. If you have the position in the
   *     original data source, you will need to call {@link #getAdjustedPosition} to get this value.
   * @since 1.0
   */
  void removeItem(int position);

  /**
   * Inserts a content row at the given position, adjusting ad positions accordingly.
   * <p>
   * This method is useful if you are inserting items in your underlying data source and wish to
   * move the positions of the ads rather than keeping them at their current positions relative
   * to the new data.
   *
   * @param position The position at which to add an item. If you have the position in the
   *     original data source, you will need to call {@link #getAdjustedPosition} to get this value.
   * @since 1.0
   */
  void addItem(int position);
}
