package com.namomedia.android;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/**
 * Targeting information used to request ads from the Namo server.
 * <p>
 * Targeting information is used to return ads that are more relevant to your users and can
 * result in better click through and cost per impression rates.
 *
 * @since 1.0
 */
public class NamoTargeting {

  /**
   * Gender values for improved ad targeting.
   *
   * @since 1.0
   */
  public static enum Gender {
    /** The male persuasion. */
    MALE,
    /** The female persuasion. */
    FEMALE,
    /** Male or female? What? */
    UNKNOWN
  }

  /**
   * Relationship status values for improved ad targeting.
   *
   * @since 1.0
   */
  public static enum RelationshipStatus {
    /** I can't believe he's single. */
    SINGLE,
    /** I can't go out with you -  I'm in a relationship. */
    IN_RELATIONSHIP,
    /** You didn't hang up - Does that mean we're engaged? */
    ENGAGED,
    /** In the second place, I'm still married. */
    MARRIED,
    /** Cause I'm in a permanently, semi, I'm in a relationship, kind of... */
    UNKNOWN
  }

  /**
   * Education values for improved ad targeting.
   *
   * @since 1.0
   */
  public static enum Education {
    /** Sir, I wrote for my high-school newspaper, sir! */
    HIGH_SCHOOL,
    /** And can you believe it? I got to go to college, too. */
    COLLEGE,
    /** It would make just enough sense to capture and fascinate graduate students. */
    GRADUATE_SCHOOL,
    /** I'm not a lawyer or a Harvard graduate or a Lutheran. */
    UNKNOWN
  }

  /**
   * Age targeting value indicating that there is no age targeting.
   */
  public final static int NO_AGE = -1;

  /**
   * Birthday targeting value indicating that there is no birthday targeting.
   */
  public final static Date NO_BIRTHDAY = new Date(Long.MAX_VALUE);

  private final Set<String> interests = new HashSet<String>();
  private final Set<String> keywords = new HashSet<String>();

  private Gender gender = Gender.UNKNOWN;
  private Education education = Education.UNKNOWN;
  private RelationshipStatus relationshipStatus = RelationshipStatus.UNKNOWN;
  private int age = NO_AGE;
  private String searchTerms = "";
  private Date birthDay = NO_BIRTHDAY;

  /**
   * Returns the gender targeting value.
   *
   * @return The gender targeting value.
   * @since 1.0
   */
  public Gender getGender() {
    return gender;
  }

  /**
   * Sets the gender targeting value.
   *
   * @param gender The new gender targeting value. Set to UNKNOWN to clear targeting.
   * @since 1.0
   */
  public void setGender(Gender gender) {
    this.gender = (gender != null ? gender : Gender.UNKNOWN);
  }

  /**
   * Returns the education targeting value.
   *
   * @return The education targeting value.
   * @since 1.0
   */
  public Education getEducation() {
    return education;
  }

  /**
   * Sets the education targeting value.
   *
   * @param education The new education targeting value. Set to UNKNOWN to clear targeting.
   * @since 1.0
   */
  public void setEducation(Education education) {
    this.education = (education != null ? education : Education.UNKNOWN);
  }

  /**
   * Returns the relationship status value.
   *
   * @return The new relationship status value.
   * @since 1.0
   */
  public RelationshipStatus getRelationshipStatus() {
    return relationshipStatus;
  }

  /**
   * Sets the relationship status value.
   *
   * @param relationshipStatus The new relationship status value. Set to UNKNOWN to clear targeting.
   * @since 1.0
   */
  public void setRelationshipStatus(RelationshipStatus relationshipStatus) {
    this.relationshipStatus =
        (relationshipStatus != null ? relationshipStatus : RelationshipStatus.UNKNOWN);
  }

  /**
   * Returns the age targeting value.
   *
   * @return The age targeting value. Returns <code>NO_AGE</code> if there is no age targeting.
   * @since 1.0
   */
  public int getAge() {
    return age;
  }

  /**
   * Returns the age targeting value.
   *
   * @param age The new age targeting value. Call {@link #clearAge} to clear the targeting.
   * @since 1.0
   */
  public void setAge(int age) {
    this.age = age;
  }

  /**
   * Clears age related targeting.
   *
   * @since 1.0
   */
  public void clearAge() {
    this.age = NO_AGE;
  }

  /**
   * Returns the search terms.
   *
   * @return The new search terms. Returns an empty string if there are no search terms.
   * @since 1.0
   */
  public String getSearchTerms() {
    return searchTerms;
  }

  /**
   * Sets the search terms. You can specify multiple terms separated by whitespace.
   *
   * @param searchTerms The new search terms.
   * @since 1.0
   */
  public void setSearchTerms(String searchTerms) {
    this.searchTerms = (searchTerms != null ? searchTerms : "");
  }

  /**
   * Returns the birthday targeting value.
   *
   * @return The birthday targeting value. Returns <code>NO_BIRTHDAY</code> if there is no birthday.
   * @since 1.0
   */
  public Date getBirthDay() {
    return birthDay;
  }

  /**
   * Sets the birthday targeting value.
   *
   * @param birthDay The new birthday targeting value. Call {@link #clearBirthDay} to clear birthday
   *     targeting.
   * @since 1.0
   */
  public void setBirthDay(Date birthDay) {
    this.birthDay = (birthDay != null ? birthDay : NO_BIRTHDAY);
  }

  /**
   * Clears birthday related targeting.
   *
   * @since 1.0
   */
  public void clearBirthDay() {
    this.birthDay = NO_BIRTHDAY;
  }

  /**
   * Returns current interests.
   *
   * @return The set of interest type targeting values. Will be an empty set if there are no values.
   * @since 1.0
   */
  public Set<String> getInterests() {
    return interests;
  }

  /**
   * Returns current targeting keywords.
   *
   * @return The set of keyword targeting values. Will be an empty set if there are no values.
   * @since 1.0
   */
  public Set<String> getKeywords() {
    return keywords;
  }

  /**
   * Adds an interest to the targeting information.
   * <p>
   * You can define your own interest strings. Some examples:
   * <ul>
   * <li>Food</li>
   * <li>Gaming</li>
   * <li>Shopping</li>
   * <li>Traveling</li>
   * <li>Autos</li>
   * <li>Education</li>
   * <li>News</li>
   * <li>Pets</li>
   * <li>Politics</li>
   * <li>Entertainment</li>
   * <li>Music</li>
   * <li>Sports</li>
   * <li><a href="http://docs.namomedia.com/interests.txt" target="_blank">More examples</a>
   </ul>
   * @param interests A list of interests.
   * @since 1.0
   */
  public void addInterests(String... interests) {
    Collections.addAll(this.interests, interests);
  }

  /**
   * Remove interests of the given type.
   *
   * @param interests A list of interests.
   * @since 1.0
   */
  public void removeInterests(String... interests) {
    this.interests.removeAll(Arrays.asList(interests));
  }

  /**
   * Clears all interests.
   *
   * @since 1.0
   */
  public void clearInterests() {
    this.interests.clear();
  }

  /**
   * Add keywords to the targeting object.
   *
   * You should only use keywords if you sell ad campaigns for your app directly and you have
   * ad groups that should only be served under specific conditions.
   *
   * @param keywords A set of keywords to target.
   * @since 1.0
   */
  public void addKeywords(String... keywords) {
    Collections.addAll(this.keywords, keywords);
  }

  /**
   * Clears all keywords.
   *
   * @since 1.0
   */
  public void clearKeywords() {
    this.keywords.clear();
  }
}
