package com.namomedia.android;

import java.util.Arrays;

/**
 * Implementation for NamoPositionAdjuster interface.
 */
class PositionAdjusterImpl implements NamoPositionAdjuster {

  private AdDataImpl[] ads = {};  // All ads, even if they aren't currently showing.

  // For each ad, store the position and original position in a sorted list. Using these lists
  // allows us to look up ad positions in log(N) time. More importantly, using unboxed ints seems
  // to make a difference in scrolling performance on Froyo, which doesn't have a concurrent
  // Garbage collector.
  //
  // An example list before and after ads.
  // firstPosition:1 spacing:1, maxAds:3
  //
  // ORIGINAL LIST    ADJUSTED LIST
  //   Item 0           Item 0
  //   Item 1           Ad
  //   Item 2           Item 1
  //   Item 3           Ad
  //   Item 4           Item 2
  //   Item 5           Ad
  //                    Item 3
  //                    Item 4
  //                    Item 5
  // adPositions is {1, 3, 5} - the ad positions in the adjusted list
  // originalPositions is {1, 2, 3} - the original position of the item following the ad.
  // adCount is 3 - in this case limited by maxAds
  private int[] adPositions = {};
  private int[] originalPositions = {};

  @Override
  public int getAdjustedCount(int originalCount) {
    if (originalCount == 0) {
      return 0;
    }
    return getAdjustedPosition(originalCount - 1) + 1;
  }

  @Override
  public int getOriginalCount(int adjustedCount) {
    if (adjustedCount == 0) {
      return 0;
    }

    // The last item will never be an ad
    return getOriginalPosition(adjustedCount - 1) + 1;
  }

  @Override
  public int getAdjustedPosition(int originalPosition) {
    int index = Arrays.binarySearch(originalPositions, originalPosition);

    // No match, ~index is the ads before this pos.
    if (index < 0) {
      return originalPosition + ~index;
    }

    // This is an ad. Since binary search doesn't properly handle dups, find the first non-ad.
    while (index < originalPositions.length && originalPositions[index] == originalPosition) {
      index++;
    }
    return originalPosition + index;
  }

  @Override
  public int getOriginalPosition(int position) {
    int index = Arrays.binarySearch(adPositions, position);

    // No match, ~index is the number of ads before this pos.
    if (index < 0) {
      return position - ~index;
    }

    // This is an ad, so there is no original position.
    return -1;
  }

  @Override
  public boolean isAd(int position) {
    int index = Arrays.binarySearch(adPositions, position);
    return index >= 0;
  }

  @Override
  public AdDataImpl getAdData(int position) {
    int index = Arrays.binarySearch(adPositions, position);
    if (index >= 0) {
      return ads[index];
    }
    return null;
  }

  void clearAds() {
    this.ads = new AdDataImpl[0];
    this.adPositions = new int[0];
    this.originalPositions = new int[0];
  }

  /**
   * Sets the ads to use when adjusting positions. Performs and immediate adjustment if the
   * number of ads has changed. This method is useful to support a continuation request for new ads.
   */
  void insertAds(AdDataImpl[] newAds, int[] newPositions) {
    Preconditions.checkState(newAds.length == newPositions.length);

    // Merge our existing ads with the new ones, assuming both lists are sorted.
    int origSize = this.ads.length;
    int origIdx = 0;
    int newSize = newAds.length;
    int newIdx = 0;
    AdDataImpl[] mergedAds = new AdDataImpl[origSize + newSize];
    int[] mergedPositions = new int[origSize + newSize];
    int mergedIdx = 0;
    while (origIdx < origSize || newIdx < newSize) {
      // Pick lowest position.
      int origPos = (origIdx < origSize) ? adPositions[origIdx] : Integer.MAX_VALUE;
      int newPos = (newIdx < newSize) ? newPositions[newIdx] : Integer.MAX_VALUE;
      AdDataImpl mergeAd;
      int mergePos;
      if (origPos < newPos) {
        mergeAd = ads[origIdx];
        mergePos = origPos;
        origIdx++;
      } else {
        mergeAd = newAds[newIdx];
        mergePos = newPos;
        newIdx++;
      }

      // Skip if duplicates
      if (mergedIdx > 0 && mergePos == mergedPositions[mergedIdx - 1]) {
        continue;
      }

      // Add the element
      mergedAds[mergedIdx] = mergeAd;
      mergedPositions[mergedIdx] = mergePos;
      mergedIdx++;
    }

    // Remove extra positions. We can't use Arrays.copyOf because it doesn't exist in Java 1.5.
    this.ads  = new AdDataImpl[mergedIdx];
    System.arraycopy(mergedAds, 0, ads, 0, mergedIdx);
    this.adPositions = new int[mergedIdx];
    System.arraycopy(mergedPositions, 0, adPositions, 0, mergedIdx);
    this.originalPositions = buildOriginalPositions(adPositions);
  }

  /**
   * Internal method to get ads.
   */
  AdDataImpl[] getAds() {
    return ads;
  }

  /**
   * Internal method to get ad positions.
   */
  int[] getAdPositions() {
    return adPositions;
  }

  private int[] buildOriginalPositions(int[] adPositions) {
    // For each ad, determine the original position of next non ad row.
    int[] result = new int[adPositions.length];
    for (int i = 0; i < adPositions.length; ++i) {
      result[i] = adPositions[i] - i;
    }
    return result;
  }

  @Override
  public void addItem(int position) {
    int index = Arrays.binarySearch(adPositions, position);
    int start = index < 0 ? ~index : index;
    for (int i = start; i < adPositions.length; ++i) {
      adPositions[i] = adPositions[i] + 1;
      originalPositions[i] = originalPositions[i] - 1;
    }
  }

  private void incrementArray(int[] target, int startIndex, int amount) {
    for (int i = startIndex; i < target.length; ++i) {
      target[i] = target[i] + amount;
    }
  }

  @Override
  public void removeItem(int position) {
    int index = Arrays.binarySearch(adPositions, position);
    int start = (index < 0) ? ~index : index;  // This is the magic to get the insertion position.
    incrementArray(adPositions, start, -1);
    if (index >= 0) {  // Greater than 0 indicates there is an ad at this position.
      adPositions = removeArrayItem(adPositions, index);
      ads = removeArrayItem(ads, index);
    }
    this.originalPositions = buildOriginalPositions(adPositions);
  }

  private AdDataImpl[] removeArrayItem(AdDataImpl[] target, int index) {
    AdDataImpl[] newArray = new AdDataImpl[target.length - 1];
    System.arraycopy(target, index + 1, newArray, index, target.length - index - 1);
    return newArray;
  }

  private int[] removeArrayItem(int[]target, int index) {
    // Can't make generic because of the native data type.
    int[] newArray = new int[target.length - 1];
    System.arraycopy(target, index + 1, newArray, index, target.length - index - 1);
    return newArray;
  }
}
