package com.namomedia.android;

import android.net.Uri;
import android.text.TextUtils;

import java.util.EnumMap;
import java.util.Map;
import java.util.Set;

/**
 * Holds the parameters used to make a given Namo Media Ads request.
 */
class RequestParams {

  private EnumMap<Urls.Param, String> params;
  private EnumMap<Urls.Param, Set<String>> repeatedParams;

  RequestParams() {
    params = new EnumMap<Urls.Param, String>(Urls.Param.class);
    repeatedParams = new EnumMap<Urls.Param, Set<String>>(Urls.Param.class);
  }

  /**
   * Put a parameter in the store that should not be repeated. Subsequent calls will overwrite
   * the previous value for the key.
   */
  void put(Urls.Param key, String value) {
    Preconditions.checkArgument(!repeatedParams.containsKey(key),
        "Request already has key %s as a repeatable parameter.", key);
    params.put(key, value);
  }

  /**
   * Store a parameter that should be repeated in the request. All values for the request should
   * be passed in at the same time. Subsequent calls will overwrite the previous value for the key.
   */
  void putRepeatable(Urls.Param key, Set<String> values) {
    Preconditions.checkArgument(!params.containsKey(key),
        "Request already has key %s as a nonrepeatable parameter.", key);
    repeatedParams.put(key, values);
  }

  /**
   * Add all the request parameters to the {@link Uri.Builder}
   */
  void addToBuilder(Uri.Builder builder) {
    // Attach params to builder.
    for (Map.Entry<Urls.Param, String> entry : params.entrySet()) {
      String value = entry.getValue();
      if (!TextUtils.isEmpty(value)) {
        builder.appendQueryParameter(entry.getKey().getName(), entry.getValue());
      }
    }

    for (Map.Entry<Urls.Param, Set<String>> entry : repeatedParams.entrySet()) {
      String key = entry.getKey().getName();
      Set<String> values = entry.getValue();
      if (values != null && values.size() != 0) {
        for (String value : values) {
          if (!TextUtils.isEmpty(value)) {
            builder.appendQueryParameter(key, value);
          }
        }
      }
    }

    // TODO(nassar): This is a temporary hack for testing until I finish setTestDevices changes.
    // builder.appendQueryParameter("test_mode", "1");
  }
}
