package com.namomedia.android;

import android.os.Build;
import android.text.Layout;
import android.text.StaticLayout;
import android.text.TextUtils;
import android.widget.TextView;

/**
 * Implementation for NamoTextLoader.
 */
class TextLoaderImpl implements NamoTextLoader {
  // Used to text chars that fit.
  private final String lotsOfEms = new String(new char[100]).replace("\0", "m");

  private final String[] textValues;
  private final LoadListener loadListener;

  private ViewSizeWaiter<TextView> viewSizeWaiter;
  private boolean allowChangeFontSize;

  /**
   * Wrapper class so that we can be sure we were the one who called setTag(value). Note that
   * setTag(int, value) leaks in Gingerbread, so we don't use it.
   */
  private static class FloatTag {
    final float value;

    FloatTag(float value) {
      this.value = value;
    }
  }

  static interface LoadListener {
    void onTextLoaded(TextView textView, int emCount);
  }

  TextLoaderImpl(String[] textValues, LoadListener loadListener) {
    this.textValues = textValues;
    this.loadListener = loadListener;
  }

  @Override
  public void into(TextView textView) {
    Preconditions.checkNotNull(textView);

    // Set the lines to a reasonable range before the measuring path. This is important
    // in order to undo the setLines call inside fillSizedText.
    textView.setMinLines(1);
    textView.setMaxLines(5);
    textView.setEllipsize(null);

    viewSizeWaiter = new ViewSizeWaiter<TextView>(textView);
    viewSizeWaiter.setListener(new ViewSizeWaiter.SizeListener<TextView>() {
      @Override
      public void onHasSize(TextView view) {
        fillSizedText(view);
      }

      @Override
      public void onFinished() {
        viewSizeWaiter = null;
      }
    });
  }

  private void fillSizedText(TextView textView) {
    // Store these values, because changing textView properties invalidates the layout.
    Layout layout = textView.getLayout();
    float spacingMult = (layout == null) ? 1.0f : layout.getSpacingMultiplier();
    float spacing = (layout == null) ? 0.0f : layout.getSpacingAdd();

    // We may have stored the original text size in the TextView's tag.
    if (textView.getTag() instanceof FloatTag) {
      FloatTag floatTag = (FloatTag) textView.getTag();
      textView.getPaint().setTextSize(floatTag.value);
      textView.setTag(null);
    }

    // Find the text that fits.
    boolean hasFit = false;
    String smallest = "";
    String largestThatFits = "";
    for (String text : textValues) {
      if (TextUtils.isEmpty(text)) {
        continue;
      }
      if (smallest.length() == 0 || smallest.length() > text.length()) {
        smallest = text;
      }
      if (largestThatFits.length() < text.length()) {
        if (checkFits(textView, text, spacingMult, spacing)) {
          largestThatFits = text;
          hasFit = true;
        }
      }
    }

    // We can shrink the text size if allowChangeFontSize is true and the tag isn't being used.
    boolean canResize = allowChangeFontSize && textView.getTag() == null;
    if (canResize && !hasFit && smallest.length() > 0) {
      float origTextSize = textView.getTextSize();

      // Allow 2/3 the current down to 10px.
      int newTextSize = normalizeSize((int)origTextSize) - 2;
      int minTextSize = normalizeSize(newTextSize * 2 / 3);
      while (newTextSize > minTextSize) {
        textView.getPaint().setTextSize(newTextSize);
        if (checkFits(textView, smallest, spacingMult, spacing)) {
          hasFit = true;
          break;
        }
        newTextSize -= 2;
      }
      // Store for later.
      textView.setTag(new FloatTag(origTextSize));
    }

    // If we still don't fit, set the line number so that we don't show partial lines.
    // TODO(nassar): If the text didn't fit, also send that info to the server.
    if (!hasFit && smallest.length() > 0) {
      int lines = linesThatFit(textView, smallest, spacingMult, spacing);
      textView.setLines(Math.max(lines, 1));
      // Pre ICS, a bug with ellipsis will render the text at only 1 line.
      if (Build.VERSION.SDK_INT >= 14) {
        textView.setEllipsize(TextUtils.TruncateAt.END);
      }
    }

    textView.setText(largestThatFits.length() > 0 ? largestThatFits : smallest);

    // Record some layout info.
    if (loadListener != null) {
      int emCount = charsThatFit(textView, lotsOfEms, spacingMult, spacing);
      loadListener.onTextLoaded(textView, emCount);
    }
  }

  private int normalizeSize(int textSize) {
    // Text tends to look better at even text sizes. Drop the last bit.
    return Math.max((textSize >> 1) << 1, 10);  // Min size of 10px
  }

  private int charsThatFit(TextView textView, String text, float spacingMult, float spacing) {
    int height = textView.getMeasuredHeight()
        - textView.getCompoundPaddingBottom() - textView.getCompoundPaddingTop();

    StaticLayout layout = createLayout(textView, text, spacingMult, spacing);
    for (int i = 0; i < layout.getLineCount(); ++i) {
      if (layout.getLineBottom(i) > height) {
        return (i == 0) ? 0 : layout.getLineEnd(i - 1);
      }
    }
    return 0;
  }

  private int linesThatFit(TextView textView, String text, float spacingMult, float spacing) {
    int height = textView.getMeasuredHeight()
        - textView.getCompoundPaddingBottom() - textView.getCompoundPaddingTop();

    StaticLayout layout = createLayout(textView, text, spacingMult, spacing);
    for (int i = 0; i < layout.getLineCount(); ++i) {
      if (layout.getLineBottom(i) > height) {
        return i;
      }
    }
    return 0;
  }

  private boolean checkFits(TextView textView, String text, float spacingMult, float spacing) {
    // Store some useful values.
    int height = textView.getMeasuredHeight()
        - textView.getCompoundPaddingBottom() - textView.getCompoundPaddingTop();
    StaticLayout layout = createLayout(textView, text, spacingMult, spacing);
    return layout.getHeight() <= height;
  }

  private StaticLayout createLayout(
      TextView textView, String text, float spacingMult, float spacing) {
    int width = textView.getMeasuredWidth()
        - textView.getCompoundPaddingRight() - textView.getCompoundPaddingLeft();

    // Use a static layout to measure since measuring the TextPaint directly is buggy.
    return new StaticLayout(
        text, textView.getPaint(), width, Layout.Alignment.ALIGN_NORMAL,
        spacingMult, spacing,
        /* include padding */ true);
  }

  @Override
  public NamoTextLoader allowChangeFontSize(boolean allowChangeFontSize) {
    this.allowChangeFontSize = allowChangeFontSize;
    return this;
  }
}
