package com.namomedia.android;

import android.view.View;
import android.view.ViewTreeObserver;

import java.lang.ref.WeakReference;

/**
 * Helper class to listen for when a view has a size.
 */
class ViewSizeWaiter<T extends View> {

  /**
   * Callback interface to indicate when the view has a size.
   */
  static interface SizeListener<T> {

    /**
     * This is the size at which the view is getting ready to draw.
     */
    void onHasSize(T view);

    /**
     * Finished listening to the size.
     */
    void onFinished();
  }

  private final WeakReference<T> viewRef;

  ViewSizeWaiter(T view) {
    Preconditions.checkNotNull(view);

    this.viewRef = new WeakReference<T>(view);
  }

  /**
   * Sets a listener. The listener will be called on the next view tree drawing pass
   * where the view has a size.
   */
  void setListener(final SizeListener<T> listener) {
    View view = viewRef.get();
    if (view == null) {
      listener.onFinished();
      return;
    }

    // Defer the fetch until the next time the image draws. Even if the image has a size, we
    // can't fetch immediately because the image size might be dependent on other dynamically
    // sized views in the layout.
    view.getViewTreeObserver().addOnPreDrawListener(new ViewTreeObserver.OnPreDrawListener() {
      @Override
      public boolean onPreDraw() {
        T sizedView = viewRef.get();
        if (sizedView == null) {
          listener.onFinished();
          return true;
        }

        ViewTreeObserver treeObserver = sizedView.getViewTreeObserver();
        if (!treeObserver.isAlive() || !hasSize(sizedView)) {
          return true;
        }

        sizedView.getViewTreeObserver().removeOnPreDrawListener(this);
        listener.onHasSize(sizedView);
        listener.onFinished();
        return true;
      }
    });
  }

  private boolean hasSize(View view) {
    return view.getWidth() > 0 || view.getHeight() > 0;
  }
}
